@@ -3,17 +3,17 @@
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
+import copy
 import os
 import shutil
 import subprocess
 import tempfile
-import unittest
 
 import dulwich.repo
 import pytest
 
 import swh.loader.git.converters as converters
-from swh.model.hashutil import bytehex_to_hash, hash_to_bytes
+from swh.model.hashutil import bytehex_to_hash, hash_to_bytehex, hash_to_bytes
 from swh.model.model import (
     Content,
     ObjectType,
@@ -90,6 +90,7 @@ class SWHTag:
 
     def __init__(
         self,
+        sha,
         name,
         type_name,
         target,
@@ -100,6 +101,7 @@ class SWHTag:
         message,
         signature,
     ):
+        self._sha = sha
         self.name = name
         self.type_name = type_name
         self.object = SWHObjectType(target_type), target
@@ -111,16 +113,17 @@ class SWHTag:
         self._tag_timezone_neg_utc = False
 
     def sha(self):
-        from hashlib import sha1
+        class hasher:
+            def digest():
+                return self._sha
 
-        return sha1()
+        return hasher
 
 
 @pytest.mark.fs
-class TestConverters(unittest.TestCase):
+class TestConverters:
     @classmethod
-    def setUpClass(cls):
-        super().setUpClass()
+    def setup_class(cls):
         cls.repo_path = tempfile.mkdtemp()
 
         bundle = os.path.join(TEST_DATA, "git-repos", "example-submodule.bundle")
@@ -161,7 +164,29 @@ class TestConverters(unittest.TestCase):
             length=124,
             status="visible",
         )
-        self.assertEqual(content, expected_content)
+        assert content == expected_content
+
+    def test_corrupt_blob(self, mocker):
+        # has a signature
+        sha1 = hash_to_bytes("28c6f4023d65f74e3b59a2dea3c4277ed9ee07b0")
+
+        blob = copy.deepcopy(self.repo[hash_to_bytehex(sha1)])
+
+        class hasher:
+            def digest():
+                return sha1
+
+        blob._sha = hasher
+
+        converters.dulwich_blob_to_content(blob)
+        converters.dulwich_blob_to_content_id(blob)
+
+        sha1 = hash_to_bytes("1234" * 10)
+
+        with pytest.raises(converters.HashMismatch):
+            converters.dulwich_blob_to_content(blob)
+        with pytest.raises(converters.HashMismatch):
+            converters.dulwich_blob_to_content_id(blob)
 
     def test_convertion_wrong_input(self):
         class Something:
@@ -169,16 +194,26 @@ class TestConverters(unittest.TestCase):
 
         m = {
             "blob": converters.dulwich_blob_to_content,
-            "blob2": converters.dulwich_blob_to_content_id,
             "tree": converters.dulwich_tree_to_directory,
             "commit": converters.dulwich_tree_to_directory,
             "tag": converters.dulwich_tag_to_release,
         }
 
         for _callable in m.values():
-            with self.assertRaises(ValueError):
+            with pytest.raises(ValueError):
                 _callable(Something())
 
+    def test_corrupt_tree(self):
+        # has a signature
+        sha1 = b"f0695c2e2fa7ce9d574023c3413761a473e500ca"
+        tree = copy.deepcopy(self.repo[sha1])
+        converters.dulwich_tree_to_directory(tree)
+
+        del tree._entries[next(iter(tree._entries))]
+
+        with pytest.raises(converters.HashMismatch):
+            converters.dulwich_tree_to_directory(tree)
+
     def test_commit_to_revision(self):
         sha1 = b"9768d0b576dbaaecd80abedad6dfd0d72f1476da"
 
@@ -214,7 +249,7 @@ class TestConverters(unittest.TestCase):
             synthetic=False,
         )
 
-        self.assertEqual(revision, expected_revision)
+        assert revision == expected_revision
 
     def test_commit_to_revision_with_extra_headers(self):
         sha1 = b"322f5bc915e50fc25e85226b5a182bded0e98e4b"
@@ -253,6 +288,23 @@ class TestConverters(unittest.TestCase):
 
         assert revision == expected_revision
 
+    @pytest.mark.parametrize(
+        "attribute", ["_message", "_encoding", "_author", "_gpgsig"]
+    )
+    def test_corrupt_commit(self, attribute):
+        # has a signature
+        sha1 = b"322f5bc915e50fc25e85226b5a182bded0e98e4b"
+        commit = copy.deepcopy(self.repo[sha1])
+        converters.dulwich_commit_to_revision(commit)
+        setattr(commit, attribute, b"abcde")
+        with pytest.raises(converters.HashMismatch):
+            converters.dulwich_commit_to_revision(commit)
+
+        if attribute == "_gpgsig":
+            setattr(commit, attribute, None)
+            with pytest.raises(converters.HashMismatch):
+                converters.dulwich_commit_to_revision(commit)
+
     def test_commit_to_revision_with_extra_headers_mergetag(self):
         sha1 = b"3ab3da4bf0f81407be16969df09cd1c8af9ac703"
 
@@ -295,7 +347,7 @@ class TestConverters(unittest.TestCase):
 
     def test_author_line_to_author(self):
         # edge case out of the way
-        with self.assertRaises(TypeError):
+        with pytest.raises(TypeError):
             converters.parse_author(None)
 
         tests = {
@@ -318,12 +370,14 @@ class TestConverters(unittest.TestCase):
 
         for author in sorted(tests):
             parsed_author = tests[author]
-            self.assertEqual(parsed_author, converters.parse_author(author))
+            assert parsed_author == converters.parse_author(author)
 
     def test_dulwich_tag_to_release_no_author_no_date(self):
-        target = b"641fb6e08ddb2e4fd096dcf18e80b894bf"
+        sha = hash_to_bytes("f6e367357b446bd1315276de5e88ba3d0d99e136")
+        target = b"641fb6e08ddb2e4fd096dcf18e80b894bf7e25ce"
         message = b"some release message"
         tag = SWHTag(
+            sha=sha,
             name=b"blah",
             type_name=b"tag",
             target=target,
@@ -342,7 +396,7 @@ class TestConverters(unittest.TestCase):
         expected_release = Release(
             author=None,
             date=None,
-            id=b"\xda9\xa3\xee^kK\r2U\xbf\xef\x95`\x18\x90\xaf\xd8\x07\t",
+            id=sha,
             message=message,
             metadata=None,
             name=b"blah",
@@ -351,11 +405,12 @@ class TestConverters(unittest.TestCase):
             target_type=ObjectType.REVISION,
         )
 
-        self.assertEqual(actual_release, expected_release)
+        assert actual_release == expected_release
 
     def test_dulwich_tag_to_release_author_and_date(self):
+        sha = hash_to_bytes("fc1e6a4f1e37e93e28e78560e73efd0b12f616ef")
         tagger = b"hey dude <hello@mail.org>"
-        target = b"641fb6e08ddb2e4fd096dcf18e80b894bf"
+        target = b"641fb6e08ddb2e4fd096dcf18e80b894bf7e25ce"
         message = b"some release message"
 
         import datetime
@@ -363,6 +418,7 @@ class TestConverters(unittest.TestCase):
         date = datetime.datetime(2007, 12, 5, tzinfo=datetime.timezone.utc).timestamp()
 
         tag = SWHTag(
+            sha=sha,
             name=b"blah",
             type_name=b"tag",
             target=target,
@@ -389,7 +445,7 @@ class TestConverters(unittest.TestCase):
                 offset=0,
                 timestamp=Timestamp(seconds=1196812800, microseconds=0,),
             ),
-            id=b"\xda9\xa3\xee^kK\r2U\xbf\xef\x95`\x18\x90\xaf\xd8\x07\t",
+            id=sha,
             message=message,
             metadata=None,
             name=b"blah",
@@ -398,14 +454,16 @@ class TestConverters(unittest.TestCase):
             target_type=ObjectType.REVISION,
         )
 
-        self.assertEqual(actual_release, expected_release)
+        assert actual_release == expected_release
 
     def test_dulwich_tag_to_release_author_no_date(self):
         # to reproduce bug T815 (fixed)
+        sha = hash_to_bytes("41076e970975122dc6b2a878aa9797960bc4781d")
         tagger = b"hey dude <hello@mail.org>"
-        target = b"641fb6e08ddb2e4fd096dcf18e80b894bf"
+        target = b"641fb6e08ddb2e4fd096dcf18e80b894bf7e25ce"
         message = b"some release message"
         tag = SWHTag(
+            sha=sha,
             name=b"blah",
             type_name=b"tag",
             target=target,
@@ -428,7 +486,7 @@ class TestConverters(unittest.TestCase):
                 name=b"hey dude",
             ),
             date=None,
-            id=b"\xda9\xa3\xee^kK\r2U\xbf\xef\x95`\x18\x90\xaf\xd8\x07\t",
+            id=sha,
             message=message,
             metadata=None,
             name=b"blah",
@@ -437,12 +495,14 @@ class TestConverters(unittest.TestCase):
             target_type=ObjectType.REVISION,
         )
 
-        self.assertEqual(actual_release, expected_release)
+        assert actual_release == expected_release
 
     def test_dulwich_tag_to_release_signature(self):
-        target = b"641fb6e08ddb2e4fd096dcf18e80b894bf"
+        target = b"641fb6e08ddb2e4fd096dcf18e80b894bf7e25ce"
         message = b"some release message"
+        sha = hash_to_bytes("46fff489610ed733d2cc904e363070dadee05c71")
         tag = SWHTag(
+            sha=sha,
             name=b"blah",
             type_name=b"tag",
             target=target,
@@ -461,7 +521,7 @@ class TestConverters(unittest.TestCase):
         expected_release = Release(
             author=None,
             date=None,
-            id=b"\xda9\xa3\xee^kK\r2U\xbf\xef\x95`\x18\x90\xaf\xd8\x07\t",
+            id=sha,
             message=message + GPGSIG,
             metadata=None,
             name=b"blah",
@@ -470,4 +530,33 @@ class TestConverters(unittest.TestCase):
             target_type=ObjectType.REVISION,
         )
 
-        self.assertEqual(actual_release, expected_release)
+        assert actual_release == expected_release
+
+    @pytest.mark.parametrize("attribute", ["name", "message", "signature"])
+    def test_corrupt_tag(self, attribute):
+        # has a signature
+        sha = hash_to_bytes("46fff489610ed733d2cc904e363070dadee05c71")
+        target = b"641fb6e08ddb2e4fd096dcf18e80b894bf7e25ce"
+        message = b"some release message"
+        tag = SWHTag(
+            sha=sha,
+            name=b"blah",
+            type_name=b"tag",
+            target=target,
+            target_type=b"commit",
+            message=message,
+            signature=GPGSIG,
+            tagger=None,
+            tag_time=None,
+            tag_timezone=None,
+        )
+        converters.dulwich_tag_to_release(tag)
+
+        setattr(tag, attribute, b"abcde")
+        with pytest.raises(converters.HashMismatch):
+            converters.dulwich_tag_to_release(tag)
+
+        if attribute == "signature":
+            setattr(tag, attribute, None)
+            with pytest.raises(converters.HashMismatch):
+                converters.dulwich_tag_to_release(tag)
